###############################################################
######### Quantitative methods III: Intoduction to R  #########
######### Rosario Barone                              #########
######### email: rosario.barone@uniroma2.it           #########
###############################################################

# Exploratory analysis of a dataset-----------------------------------------

data <- read.csv("caschool.csv",header=T,dec=".",sep=",")

str(data)
summary(data)

# Let's focus on the following two variables:

# str = Student Teacher Ratio
# testscr = average score in reading and math tests
# (Score in the reading test + Score in the math test) / 2
# These are two quantitative variables, so our exploratory analysis
# is based on the calculation of position and variability indices
# Let's first look at a summary:

summary(data$str) # the distribution has a slight negative asymmetry
summary(data$testscr)

# As can be seen from the summary, the averages are equal to:

m.str = mean(data$str); m.str
m.testscr = mean(data$testscr); m.testscr

# Variability:
# We calculate the standard deviation

sd.str = sd(data$str); sd.str
sd.testscr = sd(data$testscr); sd.testscr

# We calculate the coefficient of variation (or relative standard deviation).
# The CV is particularly useful when you want
# compare the dispersion around the mean of data having different
# units of measurement or having different ranges of variation.

cv.str = sd(data$str)/abs(mean(data$str)); cv.str
cv.testscr = sd(data$testscr)/abs(mean(data$testscr)); cv.testscr


# Let's calculate the quantiles of Table 4.1:

q.str = quantile(data$str, probs = c(0.10, 0.25, 0.40, 0.50, 0.60, 0.75, 0.90)); q.str
q.testscr = quantile(data$testscr, probs = c(0.10, 0.25, 0.40, 0.50, 0.60, 0.75, 0.90)); q.testscr

tab=rbind(c(m.str,sd.str,q.str),c(m.testscr,sd.testscr,q.testscr))

round(tab,1) 

# For a graphical interpretation of what we have done above, let's use the boxplot

boxplot(data$str, main=" student / teacher ")
boxplot(data$testscr, main="Average test score")

# Now let's look at the relationship between the two variables
plot(data$str,data$testscr, xlim=c(10,30), ylim=c(600,720),
     pch = 20, xlab=" student / teacher ", 
     ylab="Average test score", main="Scatter plot")

# Correlation coefficient between the two variables:
cor(data$str,data$testscr)
#Alternative
cov(data$str,data$testscr)/(sd.str*sd.testscr)


# Simple Linear Regression --------------------------------------------

# The correlation index shows a negative relationship between average test score  and student/teacher.
# So let's see how to describe this relationship through the regression model:
# y = beta0 + beta1 * x + e


mod=lm(data$testscr~data$str) 
str(mod)

# We can use the "summary" function for an overview on the estimation of the regression model
summary(mod)


#Alternative calculation of t-statistics

(mod$coefficients[1]-0)/9.4675
(mod$coefficients[2]-0)/0.4798


# EMPIRICAL RULE: if the p-value <level of significance (alpha), then
# I reject the null hypothesis in favor of the alternative hypothesis (H1)
# if p-value> alpha, then I don't reject the null hypothesis (H0)


# In the second part of the summary output we find the standard error of
# residuals, which is an estimator of the standard deviation of the error of
# regression; it is a measure of the dispersion of the observations around
# to the regression line.

n=420
v=sum(mod$residuals^2)/(n-2) 
sqrt(v)

# We also find the R square, which represents the fraction of the variance
# sample of "y " explained by "x "
# R ^ 2 = 1 - Sum (r_hat [i] ^ 2) / Sum ((y [i] - y *) ^ 2) where y * is the mean of y [i]
# = 1 - (SSR / TSS)
# or, R ^ 2 = ESS / RSS
# then ESS = TSS - SSR
# where ESS = sum ((y_hat [i] - y *) ^ 2), sum of squares explained
# where RSS = sum ((y [i] - y *) ^ 2), sum of the total squares
# where SSR = sum (r_hat [i] ^ 2), sum of squares of residuals
# The R ^ 2 takes values between 0 and 1; if it has a high value the model is suitable
#to explain the variability of y [i].
# if the value is low, the covariate
# in question does not capture the variability of y [i], so to predict
# better y you should add covariates.
# We calculate the R ^ 2 manually:

1-(sum(mod$residuals^2)/sum((data$testscr-mean(data$testscr))^2))


# Now let's see what the names of the contained subelements are
# in the "mod" object

names(mod) 

# Basically, we are interested in the coefficients:

mod$coefficients

# This means that the regression line will have the equation:
# y = 698.93 - 2.28 * x (Testscore = 698.93 - 2.28 * STR)
# The slope of -2.28 indicates a one-unit increase in the ratio
# students / teachers is on average associated with a reduction of 2.28
# points in the average test score.

# Let's see how the estimators of the two coefficients are calculated

beta_hat_1=cov(data$str,data$testscr)/var(data$str); beta_hat_1
beta_hat_0=mean(data$testscr)- beta_hat_1*mean(data$str); beta_hat_0

# Confidence intervals for the model coefficients --------------------

# The 95% confidence intervals (which is the default value) are:

confint(mod)

# 90% confidence intervals:

confint(mod, level=0.9)

# A 0.95 level confidence interval for beta1 has two definitions:

# - a range that has a 0.95 chance of holding the true value
# of beta1; that is, if we repeat the experiment 100 times, 95 times the interval
# contains the true value of beta1

# - the set of values that cannot be rejected using a hypothesis test
# bilateral with a significance level of 5%, that is, if we reject
# H0 with significance level 0.05, hence the interval
# of confidence level 0.95 will not contain 0 (i.e., beta below H0)


# Draw the line on the graph ------------------------------------------

plot(data$str,data$testscr,pch = 20, xlim=c(10,30), ylim=c(600,720),
     xlab="student / teacher ", 
     ylab="Average test score", main="Scatter plot")
abline(coefficients(mod),col=2,lwd=1)
# anche: abline(mod,col=2,lwd=2)


# Heteroskedasticity ----------------------------------------------- -----

# Now look at the scatter plot between the explanatory variable
# (our x) and the residuals of the estimated model, to see if there is a relationship.
# You can extract the residuals from the estimated model (mod)
# via "mod $ residuals"

plot(data$str, mod$residuals, main = "Scatter plot", 
     pch=20, xlim=c(10,30), ylim=c(-60,60),
     xlab="student / teacher", ylab="residuals")

#it seems to be everithing ok, no correlation!
cor(data$str, mod$residuals)

# In the case of homoskedasticity (hypothesis of the Gauss-Markov theorem), the
# errors are independent of X, and as such they are also uncorrelated,
# as seen from the previous command line.
# This means that E (e | X) = 0 as well as E (e) = 0
# The average of the error is zero; you can see it roughly below:

hist(mod$residuals)

# Independence implies uncorrelation, but the reverse is not true.

# If in the previous scatterplot there had been systematic
# (if we had been able to recognize some trend) then we would have
# had heteroskedasticity.



# Predicting a new value ------------------------------------------

# Suppose we want to predict the average test result for a class
# containing a number of students equal to 20 times the number of teachers
# STR = 20
# y_hat = b0 + b1 * 20, where:
# y_hat = predicted value of y
# b0 = predicted value of beta0
# b1 = predicted value of beta1

y=coefficients(mod)[1]+coefficients(mod)[2]*20; y

# we can observe that the variation of a unit in the ratio
# students / teachers, generates a decrease in the average score equal to 2.28

y.=coefficients(mod)[1]+coefficients(mod)[2]*21; y.
y-y.


# Robust estimate of the standard error --------------------------------------

# Recall: the "e" error is homoskedastic if the variance of the distribution
# "e [i]" conditional on "x [i]" is constant for i = 1, ..., n, and therefore does not depend
# from x [i];

# Robustness: small variations in population distribution
# involve small variations in the sample distribution.

# Now, we want to get robust standard errors with respect to the case of
# heteroskedasticity (i.e. the variance of the conditional errors on X
# varies over time), in this regard, we will use the AER library.
# If the latter is not already installed, just use the following command:

install.packages('AER')
library(AER)

coeftest(mod, vcov=vcovHC(mod, type="HC1"))

?coeftest
?vcovHC
# let's compare the above result with the one obtained previously, that is:
summary(mod)

# To construct the confidence interval of the angular coefficient (b1)
# 95% (using robust standard error), some
# elements from the output of "coeftest", therefore, preliminarily, the
# we assign a name, in this case "ct".

ct <- coeftest(mod, vcov=vcovHC(mod, type="HC1"))

str(ct)

# "ct" is a matrix with two rows and four columns, and we can extract
# elements from the same with the usual notation of square brackets.
# Unlike "mod", "ct" is not a modeled model, so running
# "confint (ct)" we would be wrong.
# We have to manually construct the confidence interval:
# [beta1_hat - 1.96 * SE (beta1_hat), beta1_hat + 1.96 * SE (beta1_hat)]


inf=ct[2,1]-qnorm(0.975)*ct[2,2]
sup=ct[2,1]+qnorm(0.975)*ct[2,2]
c(inf,sup)

confint(mod)[2,]

# Note that using robust standard errors, the  inferentials do not change, but the values 
# of the t-statistic, of the p-value and the confidence interval are slightly different.
# In general, use robust standard errors in the case of homoskedasticity it is not a problem;
# vice versa, to use classic standard errors in the case
# of heteroskedasticity is inappropriate; in fact, the t-statistic, calculated
# using classic standard error, has no standard normal distribution
# not even in large samples.


# Model with dummy explanatory variable ---------------------------------

# The regressor is a binary variable

# Let's create an indicator function:
# that is, a function that is 1 if the student / teacher ratio <20
# and it is 0 elsewhere (therefore, when the latter is equal to 20 or greater)

dummy.str = I(data$str<20)

mod1 <- lm(data$testscr ~ dummy.str)
summary(mod1)

# The scatter plot between the binary variable and
# the dependent variable (average test score).


plot(dummy.str, data$testscr, xlim=c(-1,2), ylim=c(600,720), pch=20,  ylab="test score")
abline(coefficients(mod1), lwd=3, col=2)

# So, when D [i] is the regressor, we have
# the following regression model:
# y [i] = beta0 + beta1 * D [i] + e [i] where i = 1, ..., n

# We split into two different cases, so we have that:
# y [i] = beta0 + e [i] if D [i] = 0
# y [i] = beta0 + beta1 + e [i] if D [i] = 1

# The conditional mean of the error is 0, regardless of how much
# holds D [i], that is: E (e | D [i]) = 0
# Hence, the conditional expectation of Y [i] where D [i] = 1 is equal to:
# E (Y [i] | D [i] = 1) = beta0 + beta1
# Instead, the conditional expectation of Y [i] where D [i] = 0 is equal to:
# E (Y [i] | D [i] = 0) = beta0
# Rather than as an angular coefficient beta1 can be seen
# as the difference between the conditional means.
# So if beta1 is significantly different from zero, that means
# having a student / teacher ratio of less than 20 results in an average of
# test results different from what you get when the same ratio
# is greater than or equal to 20.
# beta1 is significantly different from 0 since the test statistic
# equal to 4 is greater than 1.96 (i.e. the value it puts in front of itself
# lo 0.025 of the probability mass)

tapply(data$testscr, dummy.str , mean)

# 95% confidence intervals of model parameters

confint(mod1) # the confidence interval excludes 0

plot(data$el_pct,data$testscr,pch = 20, xlim=c(0,90), ylim=c(600,720),
     xlab="Percentage of students who speak english", 
     ylab="test score", main="scatter plot ")
cor(data$el_pct,data$testscr)
model=lm(data$testscr~data$el_pct)
summary(model)
coefficients(model)[1]+coefficients(model)[2]*20

# Multiple linear regression ---------------------------------------------- ----------------

# This time we regress the response variable (testscr) to
# multiple explanatory variables:
# str (student / teacher ratio)
# el_pct (percentage of students learning English)

# We extract these variables and put them in a reduced dataset:
data.sub=data[,c(11,14,16)]
# Brief exploratory analysis
pairs(data.sub)
cor(data.sub)

# Let's move on to the Multiple Linear Regression model;
# we use the same command as for simple linear regression,
# by modifying the formula, that is, adding other explanationatory variables:

m2=lm(data$testscr ~ data$str + data$el_pct)

summary(m2)

# Comments on m2 output:
# with a significance level of 5%, all coefficients are
# meaningful.
# The F statistic causes the null hypothesis to be rejected as proof
# that much of the variance of the scores is explained
# from the explanatory variables, as can be seen from the non-low value
# of the R ^ 2.


# Diagnostics ------------------------------------------------ -------------

# Let's look at the prediction plots of the response variable
# with respect to the residuals of 'm2' and their absolute values

plot(fitted(m2), residuals(m2), xlab="predicted Y", ylab="residuals",
     main="residuals vs  predicted Y")
abline(h=0,col=2,lwd=2)
plot(fitted(m2), abs(residuals(m2)), xlab="predicted Y", ylab="absolute value of the residuals",
     main="absolute value of the residuals vs predicted Y")


# Some trend is attributable to this point cloud: therefore
# there is heteroskedasticity

plot(data$str, residuals(m2), xlab="students/teacher", ylab="residuals",
     main="residuals vs student/teacher")

plot(data$el_pct, residuals(m2), xlab="Percentage of non english mother tongue students", ylab="residuals", main="residuals vs percentage of non english mother tongue students")

# Having heteroskedasticity implies robust estimates for errors
# standard

# The residuals distribute according to a Normal. Let's check it out:

res.mod=residuals(m2)

qqnorm(res.mod)
qqline(res.mod)


# Robust estimates of standard errors --------------------------------------------

library(AER)
coeftest(m2, vcov = vcovHC(m2, type = "HC1"))

# 0.95 level confidence intervals for model parameters;
# Robust standard errors are used

ct <- coeftest(m2, vcov = vcovHC(m2, type = "HC1"))

l.95=ct[1:3,1]-qnorm(0.995)*ct[1:3,2]
u.95=ct[1:3,1]+qnorm(0.995)*ct[1:3,2]

cbind(l.95,u.95)

#  0.99 confidence level

l.99=ct[2:3,1]-qnorm(0.995)*ct[2:3,2]
u.99=ct[2:3,1]+qnorm(0.995)*ct[2:3,2]

cbind(l.99,u.99)

c=cor(data$str, data$el_pct);c
cor_coeff=-c; cor_coeff # confidence region


# Graphic representation -----------------------------------------------

library(scatterplot3d)
s3d <- scatterplot3d(data$str, data$el_pct, data$testscr, color="red",pch=19,angle=10)
s3d$plane3d(m2, col="blue", lty = "solid", lwd=2)
# Note that both coefficients are negative

# Instead, if a coefficient is positive, the regression plan results:

m2.1=lm(data$testscr ~ data$expn_stu + data$el_pct)
s3d.1 <- scatterplot3d(data$expn_stu, data$el_pct, data$testscr, color="red",pch=19,angle=10)
s3d.1$plane3d(m2.1, col="blue", lty = "solid",lwd=2)


# Collinearity ----------------------------------------------- -------------

# We define a new variable as a linear combination of another

el_pct_rel <- data$el_pct/100 # this new variable has values between 0 and 1
# and is perfectly correlated with "data $ el_pct" (perfect collinearity)

# What happens if we include both in the same model?

mod3_coll=lm(data$testscr ~ data$str + data$el_pct + el_pct_rel)

summary(mod3_coll)

cor(data$el_pct , el_pct_rel) 

############################################################################################
############################################################################################
############################################################################################
# Estimating "by hand" the multivariate-regression coefficients:
# y<-b*X+e 
# where b is the vector of coeffients (b0,b1,b2) and b0 is the intercept.

# 1) Define a vector y
y<-data$testscr
# 2) Define the matrix of covariates, in which the first column vector will be equal to 1
# in order to allow the estimation of the inetrcept
X<-cbind(rep(1,length(y)),data$str , data$el_pct)

b<-t(y)%*%X%*%solve(t(X)%*%X)

#Now we define the model as in md3_coll. The vector of regression coefficients
# is now (b0, b1, b2, b3), where b0 is again the intercept.

# We re-write the matrix of the covariates by adding the variable el_pct_rel
X<-cbind(rep(1,length(y)),data$str , data$el_pct , el_pct_rel)

b<-t(y)%*%X%*%solve(t(X)%*%X)

#ERROR: Since two of the covariates are perfectly correlated,the product of matrices 
# t(X)%*%(X) is a singular  matrix and can not be inverted.
# Therefore, the estimation of the coefficients is possible only by drop 
# out of one of the perfectly correlated variables.


############################################################################################
############################################################################################
############################################################################################

# If, on the other hand, the correlation is not perfect, we have imperfect collinearity
# and this does not prevent the estimation of the coefficient.
# Imperfect collinearity indicates that two or more regressors
# are highly correlated, as is the case, for example, of students who
# are learning English and immigrant students, whose second language is
# usually English (so they tend to study it).


# Scale of variables ---------------------------------------------- ---------

expenditure <- data$expn_stu/1000 #Total expenditure per student in thousands of dollars

mod4a<- lm(data$testscr ~ data$str + data$el_pct + expenditure)
summary(mod4a)


plot(fitted(mod4a), residuals(mod4a), xlab="predicted Y", ylab="residuals",
     main="residuals vs  predicted Y")

coeftest(mod4a, vcov = vcovHC(mod4a, type = "HC1"))

# Are the coefficients all significant? No, the student / teacher ratio
# it is not'.

# Another question

# What coefficient would we have had if the additional variable on total expenditure
# per student was expressed in dollars? We would have had the same
# coefficient divided by one thousand.

mod4b<- lm(data$testscr ~ data$str + data$el_pct + data$expn_stu)
summary(mod4b)

coeftest(mod4b, vcov = vcovHC(mod4b, type = "HC1"))
