###############################################################
######### Quantitative methods III: Intoduction to R  #########
######### Rosario Barone                              #########
######### email: rosario.barone@uniroma2.it           #########
###############################################################
library(AER)

dati <- read.csv("caschool.csv")
attach(dati)
income <- avginc

# Non linear relations ---------------------------------------------- -----

# Let consider the relationship between the test result (testscr)
# and district income 
# We analyze the distribution of income
median(income)
mean(income)
hist(income)

# there is a high correlation between the two variables, proving the fact
# that the economic condition affects the average test score

cor(testscr,income)
plot(income,testscr)

# Let first consider a simple linear model 
linear.model=lm(testscr~income)
summary(linear.model)

abline(coefficients(linear.model),col=2,lwd=2)

# Most of the points lie below the line when income
# is very low and very high, but is above in the middle.
# Rather than using a straight line to describe the course of the points
# it seems more plausible to use another curve.
# A quadratic function could be fine!


# Nonlinear relationships: Quadratic covariate  -------------------------------------

# Introduce the squared term for income and compare it
# with linear model:

# First way: preliminarily create the variables to be inserted in the model
income2=income^2
quadratic.model1=lm(testscr~income+income2)
summary(quadratic.model1)
#Analogously
quadratic.model=lm(testscr~income+I(income^2))
summary(quadratic.model)

### This may be still estimated via OLS, since "quadratic" is the covariate,
### while the parameters are still linear!

# Comparison of the models
summary(quadratic.model)
summary(linear.model)


# Confrontiamo i grafici
plot(income,testscr,xlab="Income",ylab="test score")
abline(coefficients(linear.model),col=2,lwd=2)
x <- seq(min(income), max(income), by=0.05) 
lines(x, predict(quadratic.model, data.frame(income=x)), col='blue', lwd=2)

### The smaller is the "step" of the sequence, the higher is the number of points
### in which we evaluate the function, the more is the accuracy in the plot
set<-c(20,10,5,1,0.5,0.0001)
for(j in set){
plot(income,testscr,xlab="Income",ylab="test score", main = paste("Step",j))
x <- seq(min(income), max(income), by=j) 
lines(x, predict(quadratic.model, data.frame(income=x)), col='blue', lwd=3)      
}

# Robust estimates

ct<-coeftest(quadratic.model, vcov = vcovHC(quadratic.model, type = "HC1"))

#Intervallo di confidenza al 95%
li<-ct[2:3,1]-qnorm(0.975)*ct[2:3,2]
ui<-ct[2:3,1]+qnorm(0.975)*ct[2:3,2]
cbind(li,ui)


# Check the residuals:

plot(fitted(linear.model),residuals(linear.model),xlab="Fitted Y",
     ylab="Residuals",main="Linear Model")


plot(fitted(quadratic.model),residuals(quadratic.model),xlab="Fitted Y",
     ylab="Residuals",main="Quadratic Model")



# Plynomial Regression -------------------------------------

# What if instead of a quadratic relation we had other terms?
# Below is the cubic regression model

pol.mod3=lm(testscr~income+I(income^2)+I(income^3))
summary(pol.mod3)


# Residuals:

plot(fitted(pol.mod3),residuals(pol.mod3),xlab="Fitted Y",
     ylab="Residuals",main="Polynomial Model (3rd)")

# Robust estimates

ct<-coeftest(pol.mod3, vcov = vcovHC(pol.mod3, type = "HC1"))

#Intervallo di confidenza al 95%
li<-ct[2:4,1]-qnorm(0.975)*ct[2:4,2]
ui<-ct[2:4,1]+qnorm(0.975)*ct[2:4,2]
cbind(li,ui)


# Vediamo un polinomio di grado 4

pol.mod4=lm(testscr~income+I(income^2)+I(income^3)+I(income^4))
summary(pol.mod4)

# Vediamo i residui
plot(fitted(pol.mod4),residuals(pol.mod4),xlab="Fitted Y",
     ylab="Residuals",main="Polynomial Model (4th)")



# Robust estimates

ct<-coeftest(pol.mod4, vcov = vcovHC(pol.mod4, type = "HC1"))

#Intervallo di confidenza al 95%
li<-ct[2:5,1]-qnorm(0.975)*ct[2:5,2]
ui<-ct[2:5,1]+qnorm(0.975)*ct[2:5,2]
cbind(li,ui)

#Intervallo di confidenza al 99%
li<-ct[2:4,1]-qnorm(0.995)*ct[2:4,2]
ui<-ct[2:4,1]+qnorm(0.995)*ct[2:4,2]
cbind(li,ui)




# Multiple Hypothesis testing: 
# Test the fourth degree Polynomial model (H1) with vs Quadratic model (H0)

?linearHypothesis
## Function for joint test on multiple parameters in linear models:
#Define the Matrix H, with:
# number of rows = the number of parameters we want to test
# number of columns= the number of parameteres in the model
# We set 1 for the parameters to be tested
H <- rbind( c(0,0,0,1,0), c(0,0,0,0,1));H
h <- c(0,0)
linearHypothesis(pol.mod4,H,h,vcov=vcovHC(pol.mod4, type = "HC1"))
# The Null Hypothesis of a quadratic model is rejected with a 5% significance level
# against the hypothesis of a fourth degree polynomial.

#Test the fourth degree Polynomial model (H1) with vs linear model (H0)
H <- rbind(c(0,0,1,0,0), c(0,0,0,1,0), c(0,0,0,0,1));H
h <- c(0,0,0)
linearHypothesis(pol.mod4,H,h,vcov=vcovHC(pol.mod4, type = "HC1"))


# Linear-logarithmic regression -------------------------------------

# the use of the logarithmic function of a regressor allows us to grasp
# a non-linear relationship and to interpret the coefficient in terms of percentage changes

#First model:

# y=a+b*log(x)+e

mod.log=lm(testscr~log(income))
summary(mod.log)

# Residuals
plot(fitted(mod.log),residuals(mod.log),xlab="Fitted Y",
     ylab="Residuals",main="Linear-logarithmic model")

# Robust Estimates:

ct<-coeftest(mod.log, vcov = vcovHC(mod.log, type = "HC1"))

# 95% Confidence Interval
li<-ct[2,1]-qnorm(0.975)*ct[2,2]
ui<-ct[2,1]+qnorm(0.975)*ct[2,2]
cbind(li,ui)

# 99% Confidence Interval
li<-ct[2,1]-qnorm(0.995)*ct[2,2]
ui<-ct[2,1]+qnorm(0.995)*ct[2,2]
cbind(li,ui)


# Beware of interpretation!
# In the linear-logarithmic model, a change in X (income) of 1% is
# associated with a variation of Y equal to 0.01 * beta1.
# Specifically, if income varies by 1% (not of one unit),
# Y varies by 0.36 points (i.e. 0.01 * 36.42)

36.42*log(101/100)
36.42*log(11/10)

# When income is low, the increase in income has an effect
# higher on the score, compared to the case in which the income is high.


# Let's compare the models:

plot(income,testscr,xlab="Income",ylab="Average test score")
abline(coefficients(linear.model),col=2,lwd=2)
x <- seq(min(income), max(income), by=0.05)
lines(x, predict(quadratic.model, data.frame(income=x)), col=4, lwd=2)
lines(x, predict(pol.mod3, data.frame(income=x)), col=3, lwd=2)
lines(x, predict(pol.mod4, data.frame(income=x)), col=6, lwd=2)
lines(x, predict(mod.log, data.frame(income=x)), col=7, lwd=2)
legend("bottomright",legend=c("mod.lin","mod.quad","mod.pol3","mod.pol4","mod.log"),
       lwd=2,col=c(2,4,3,6,7), cex=0.5)


#Log-linear Model -----------------------------------------------------

# log(y)=a+b*x

mod.logy=lm(log(testscr)~income)
summary(mod.logy)

plot(fitted(mod.logy),residuals(mod.logy),xlab="Fitted Y",
     ylab="Residuals",main="Log-Linear Model")

# Robust estimates:

ct<-coeftest(mod.logy, vcov = vcovHC(mod.logy, type = "HC1"))

#95% Confidence Interval
li<-ct[2,1]-qnorm(0.975)*ct[2,2]
ui<-ct[2,1]+qnorm(0.975)*ct[2,2]
cbind(li,ui)

#99% Confidence Interval
li<-ct[2,1]-qnorm(0.995)*ct[2,2]
ui<-ct[2,1]+qnorm(0.995)*ct[2,2]
cbind(li,ui)


# As before, you have to pay attention to the interpretation!
# In this case we are no longer talking about the effect on the average test score,
# but of effect on the percentage change of the average test score

# In this model, a variation of one unit of X corresponds
# to a 100 * beta1% variation of Y
# In this regression the score increases by 0.28% for each
# unit of additional income.

#Log-log model -----------------------------------------------------

# log(y)= a+b*log(x)

mod.log.log=lm(log(testscr)~log(income))
summary(mod.log.log)

plot(fitted(mod.log.log),residuals(mod.log.log),xlab="Fitted Y",
     ylab="Residuals",main="Log-log model")

#Robust Estimates:

coeftest(mod.log.log, vcov = vcovHC(mod.log.log, type = "HC1"))

#95% Confidence Interval
li<-ct[2,1]-qnorm(0.975)*ct[2,2]
ui<-ct[2,1]+qnorm(0.975)*ct[2,2]
cbind(li,ui)

#99% Confidence Interval
li<-ct[2,1]-qnorm(0.995)*ct[2,2]
ui<-ct[2,1]+qnorm(0.995)*ct[2,2]
cbind(li,ui)


# In this case we are talking about percentage changes for both the explanatory
# and the response variable.
# In this model, a variation of X of 1% corresponds to a variation of Y of beta1%.
# In this case, if the income varies by 1%, the score varies by 0.055%:
# in other terms, beta1 is the elasticity of Y with respect to X.



# Let's compare the last two models

plot(income,log(testscr))
lines(x, predict(mod.logy, data.frame(income=x)), col=4, lwd=2)
lines(x, predict(mod.log.log, data.frame(income=x)), col=6, lwd=2)
legend("bottomright",legend=c("mod.logy","mod.log.log"),
       lwd=2,col=c(4,6), cex=0.5)

# Even the log-log model doesn't fit the data very well, in fact
# for lower income levels, most observations
# lie below the curve, while for average incomes, the greater
# part of the observations lie above


#Log-Polynomial model
# y<-a+ b1*log(x)+b2*log(x)^2+b3*log(x)^3

m5 <- lm(testscr ~ log(income) + I(log(income)^2) + I(log(income)^3))
summary(m5)

plot(fitted(m5),residuals(m5),xlab="Fitted Y",
     ylab="Residuals",main="Log-polynomial model")

#Robust Estimates:

coeftest(m5, vcov = vcovHC(m5, type = "HC1"))

# Test the Log-Polynomial model (H1) vs the Linear-Logarithmic model (H0)
H2 <- rbind(c(0,0,1,0), c(0,0,0,1))
h2 <- c(0,0)
linearHypothesis(m5, H2, h2, vcov=vcovHC(m5, type = "HC1"))

# We cannot reject H0 
# This model does not statistically improve the linear-logarithmic model


plot(income,testscr,xlab="Income",ylab="Average test score")
lines(x, predict(mod.log, data.frame(income=x)), col=3, lwd=2)
lines(x, predict(m5, data.frame(income=x)), col=2, lwd=2)
legend("bottomright",legend=c("mod.log","m5"),
       lwd=2,col=c(3,2), cex=0.5)


# Interaction between two dummy variables ------------------------------------

#Create a dummy variable such tath if the percentage of english native speakers
# is grater than 10%, du.eng=1, otherwise du.eng=0.
du.eng <- 1*(el_pct>=10)

# We introduce the model with different slopes and intercepts.
# The effect on the test score of a reduction in the
# students-teachers ratio depends on the fact that the percentage of non-native speakers
# is it high or low?
# To answer we use a model with different intercepts and different slopes

m6 <- lm(testscr ~ str + du.eng + I(str*du.eng))
summary(m6)

# Same slope with different intercepts
m6.1 <- lm(testscr ~ str + du.eng)
summary(m6.1)
coeftest(m6.1, vcov = vcovHC(m6.1, type = "HC1"))

# Different slopes with same intercept
m6.2 <- lm(testscr ~ str + I(str*du.eng))
summary(m6.2)
coeftest(m6.2, vcov = vcovHC(m6.2, type = "HC1"))

### Interpretation of the results of the models m6, m6.1 and m6.2 as an exercise ###

